@@ -16,10 +16,17 @@
 #    define EXTERN_C
 #endif
 
-#if defined(_WIN32)
+#if defined(_WIN32) && defined(__MINGW32__) 
+#define WINVER 0x500	/* Dirty trick, but it works. */
 #    include <windows.h>
 #    include <process.h>
 #endif
+
+#if defined(_WIN32) && !defined(__MINGW32__)
+#    include <windows.h>
+#    include <process.h>
+#endif
+
 #if defined(_MSC_VER)
 #    include <intrin.h>
 #else
@@ -28,11 +35,13 @@
 
 /* end of portability-macros section */
 
-#define DDS_VERSION		20101	/* Version 2.1.1. Allowing for 2 digit
+#define DDS_VERSION		20300	/* Version 2.3.0. Allowing for 2 digit
 					minor versions */
 
 #define PBN
 
+#define PBN_PLUS
+
 /*#define BENCH*/
 
 #include <stdio.h>
@@ -73,14 +82,14 @@
 #define NSIZE	100000
 #define WSIZE   100000
 #define LSIZE   20000
-#define NINIT	250000/*400000*/
-#define WINIT	700000/*1000000*/
+#define NINIT	250000
+#define WINIT	700000
 #define LINIT	50000
 
 #define SIMILARDEALLIMIT	5
 #define SIMILARMAXWINNODES  700000
 
-#define MAXNOOFBOARDS		20
+#define MAXNOOFBOARDS		100
 
 #define Max(x, y) (((x) >= (y)) ? (x) : (y))
 #define Min(x, y) (((x) <= (y)) ? (x) : (y))
@@ -93,6 +102,8 @@ All hand identities are given as
 0=NORTH, 1=EAST, 2=SOUTH, 3=WEST. */
 
 #define handId(hand, relative) (hand + relative) & 3
+#define CMP_SWAP(i, j) if (a[i].weight < a[j].weight)  \
+  { struct moveType tmp = a[i]; a[i] = a[j]; a[j] = tmp; } 
 
 
 struct gameInfo  {          /* All info of a particular deal */
@@ -184,7 +195,7 @@ struct pos {
 
 struct posSearchType {
   struct winCardType * posSearchPoint; 
-  __int64 suitLengths;
+  long long suitLengths;
   struct posSearchType * left;
   struct posSearchType * right;
 };
@@ -214,10 +225,16 @@ struct evalType {
   unsigned short int winRanks[4];
 };
 
+struct absRankType {
+  char rank;
+  char hand;
+};
+
 struct relRanksType {
   int aggrRanks[4];
   int winMask[4];
   char relRank[15][4];
+  struct absRankType absRank[15][4];
 };
 
 struct adaptWinRanksType {
@@ -248,9 +265,16 @@ struct boards {
   int mode[MAXNOOFBOARDS];
 };
 
+struct boardsPBN {
+  int noOfBoards;
+  struct dealPBN deals[MAXNOOFBOARDS];
+  int target[MAXNOOFBOARDS];
+  int solutions[MAXNOOFBOARDS];
+  int mode[MAXNOOFBOARDS];
+};
+
 struct solvedBoards {
   int noOfBoards;
-  int timeOut;
   struct futureTricks solvedBoard[MAXNOOFBOARDS];
 };
 
@@ -258,9 +282,7 @@ struct paramType {
   int noOfBoards;
   struct boards *bop;
   struct solvedBoards *solvedp;
-  int tstart;
-  int timeSupervision;
-  int remainTime;
+  int error;
 };
 
 struct ddTableDeal {
@@ -275,6 +297,20 @@ struct ddTableResults {
   int resTable[5][4];
 };
 
+struct parResults {
+  char parScore[2][16];	/* index = 0 is NS view and index = 1 is EW view. */
+  char parContractsString[2][128]; /* index = 0 is NS view and index = 1 
+				      is EW view. By view is here meant 
+				      which side that starts the bidding. */
+};
+
+struct par_suits_type {
+  int suit;
+  int tricks;
+  int score;
+}; 
+
+
 struct localVarType {
   int nodeTypeStore[4];
   int trump;
@@ -288,7 +324,7 @@ struct localVarType {
   int val;
   struct pos iniPosition;
   struct pos lookAheadPos; /* Is initialized for starting
-							alpha-beta search */
+			      alpha-beta search */
   struct moveType forbiddenMoves[14];
   struct moveType initialMoves[4];
   struct moveType cd;
@@ -308,18 +344,12 @@ struct localVarType {
   struct moveType bestMove[50];
   struct moveType bestMoveTT[50];
   struct winCardType temp_win[5];
-  /*int hiwinSetSize;
-  int hinodeSetSize;
-  int hilenSetSize;
-  int MaxnodeSetSize;
-  int MaxwinSetSize;
-  int MaxlenSetSize;*/
   int nodeSetSizeLimit;
   int winSetSizeLimit;
   int lenSetSizeLimit;
-  __int64 maxmem;		/* bytes */
-  __int64 allocmem;
-  __int64 summem;
+  unsigned long long maxmem;		/* bytes */
+  unsigned long long allocmem;
+  unsigned long long summem;
   int wmem;
   int nmem; 
   int lmem;
@@ -333,7 +363,6 @@ struct localVarType {
   int suppressTTlog;*/
   struct relRanksType * rel;
   struct adaptWinRanksType * adaptWins;
-  __int64 suitLengths;
   struct posSearchType *rootnp[14][4];
   struct winCardType **pw;
   struct nodeCardsType **pn;
@@ -350,28 +379,9 @@ struct localVarType {
   int lenSetSize;  /* Index with range 0 to lenSetSizeLimit */
 };
 
-#if defined(_WIN32)
-extern CRITICAL_SECTION solv_crit;
-#endif
-
 extern int noOfThreads;
 extern int noOfCores;
 extern struct localVarType localVar[MAXNOOFTHREADS];
-extern struct gameInfo game;
-extern int newDeal;
-extern struct gameInfo * gameStore;
-extern struct pos position, iniPosition, lookAheadPos;
-extern struct moveType move[13];
-extern struct movePlyType movePly[50];
-extern struct searchType searchData;
-extern struct moveType forbiddenMoves[14];  /* Initial depth moves that will be 
-					       excluded from the search */
-extern struct moveType initialMoves[4];
-extern struct moveType highMove;
-extern struct winCardType **pw;
-extern struct nodeCardsType **pn;
-extern struct posSearchType **pl;
-
 extern int * highestRank;
 extern int * counttable;
 extern struct adaptWinRanksType * adaptWins;
@@ -384,8 +394,6 @@ extern int c1[50], c2[50], c3[50], c4[50], c5[50], c6[50], c7[50],
 extern int nodeTypeStore[4];            /* Look-up table for determining if
                                         node is MAXNODE or MINNODE */
 extern int lho[4], rho[4], partner[4];                                        
-extern int trumpContract;
-extern int trump;
 extern int nodes;                       /* Number of nodes searched */
 extern int no[50];                      /* Number of nodes searched on each
                                         depth level */
@@ -414,8 +422,6 @@ extern unsigned char suitChar[4];
 extern unsigned char rankChar[15];
 extern unsigned char handChar[4];
 extern unsigned char cardRank[15], cardSuit[5], cardHand[4];
-extern unsigned char cardSuitSds[5];
-extern struct handStateType handState;
 extern int totalNodes;
 extern struct futureTricks fut, ft;
 extern struct futureTricks *futp;
@@ -431,20 +437,27 @@ extern int suppressTTlog;
 
 EXTERN_C DLLEXPORT int STDCALL SolveBoard(struct deal dl, 
   int target, int solutions, int mode, struct futureTricks *futp, int threadIndex);
-
-#ifdef PBN
-EXTERN_C DLLEXPORT int STDCALL SolveBoardPBN(struct dealPBN dlpbn, int target, 
-    int solutions, int mode, struct futureTricks *futp, int thrId);
-#endif
-
 EXTERN_C DLLEXPORT int STDCALL CalcDDtable(struct ddTableDeal tableDeal, 
   struct ddTableResults * tablep);
 
 #ifdef PBN
+EXTERN_C DLLEXPORT int STDCALL SolveBoardPBN(struct dealPBN dlpbn, int target, 
+    int solutions, int mode, struct futureTricks *futp, int thrId);
 EXTERN_C DLLEXPORT int STDCALL CalcDDtablePBN(struct ddTableDealPBN tableDealPBN, 
   struct ddTableResults * tablep);
 #endif
 
+#ifdef PBN_PLUS
+EXTERN_C DLLEXPORT int STDCALL SolveAllBoards(struct boardsPBN *bop, struct solvedBoards *solvedp);
+EXTERN_C DLLEXPORT int STDCALL CalcPar(struct ddTableDeal tableDeal, int vulnerable, 
+    struct ddTableResults * tablep, struct parResults *presp);
+EXTERN_C DLLEXPORT int STDCALL CalcParPBN(struct ddTableDealPBN tableDealPBN, 
+  struct ddTableResults * tablep, int vulnerable, struct parResults *presp);
+#endif
+
+#ifdef __linux
+EXTERN_C void InitStart(int gb_ram, int ncores);   /* For usage with ctypes in Linux. */ 
+#endif
 
 void InitStart(int gb_ram, int ncores);
 void InitGame(int gameNo, int moveTreeFlag, int first, int handRelFirst, int thrId);
@@ -452,17 +465,16 @@ void InitSearch(struct pos * posPoint, int depth,
   struct moveType startMoves[], int first, int mtd, int thrId);
 int ABsearch(struct pos * posPoint, int target, int depth, int thrId);
 void Make(struct pos * posPoint, unsigned short int trickCards[4], 
-  int depth, int thrId);
-int MoveGen(struct pos * posPoint, int depth, int thrId);
-void InsertSort(int n, int depth, int thrId);
-void UpdateWinner(struct pos * posPoint, int suit);
-void UpdateSecondBest(struct pos * posPoint, int suit);
-int WinningMove(struct moveType * mvp1, struct moveType * mvp2, int thrId);
+  int depth, int trump, struct movePlyType *mply, int thrId);
+int MoveGen(struct pos * posPoint, int depth, int trump, struct movePlyType *mply, int thrId);
+void MergeSort(int n, struct moveType *a);
+inline int WinningMove(struct moveType * mvp1, struct moveType * mvp2, int trump, int thrId);
+inline int WinningMoveNT(struct moveType * mvp1, struct moveType * mvp2, int thrId);
 int AdjustMoveList(int thrId);
 int QuickTricks(struct pos * posPoint, int hand, 
-	int depth, int target, int *result, int thrId);
-int LaterTricksMIN(struct pos *posPoint, int hand, int depth, int target, int thrId); 
-int LaterTricksMAX(struct pos *posPoint, int hand, int depth, int target, int thrId);
+	int depth, int target, int trump, int *result, int thrId);
+int LaterTricksMIN(struct pos *posPoint, int hand, int depth, int target, int trump, int thrId); 
+int LaterTricksMAX(struct pos *posPoint, int hand, int depth, int target, int trump, int thrId);
 struct nodeCardsType * CheckSOP(struct pos * posPoint, struct nodeCardsType
   * nodep, int target, int tricks, int * result, int *value, int thrId);
 struct nodeCardsType * UpdateSOP(struct pos * posPoint, struct nodeCardsType
@@ -475,14 +487,14 @@ struct nodeCardsType * BuildPath(struct pos * posPoint,
 void BuildSOP(struct pos * posPoint, int tricks, int firstHand, int target,
   int depth, int scoreFlag, int score, int thrId);
 struct posSearchType * SearchLenAndInsert(struct posSearchType
-	* rootp, __int64 key, int insertNode, int *result, int thrId);  
+	* rootp, long long key, int insertNode, int *result, int thrId);  
 void Undo(struct pos * posPoint, int depth, int thrId);
 int CheckDeal(struct moveType * cardp, int thrId);
 int InvBitMapRank(unsigned short bitMap);
 int InvWinMask(int mask);
 void ReceiveTTstore(struct pos *posPoint, struct nodeCardsType * cardsP, int target, 
   int depth, int thrId);
-int NextMove(struct pos *posPoint, int depth, int thrId); 
+int NextMove(struct pos *posPoint, int depth, struct movePlyType *mply, int thrId); 
 int DumpInput(int errCode, struct deal dl, int target, int solutions, int mode); 
 void Wipe(int thrId);
 void AddNodeSet(int thrId);
@@ -491,8 +503,8 @@ void AddWinSet(int thrId);
 
 void PrintDeal(FILE *fp, unsigned short ranks[4][4]);
 
-int SolveAllBoards4(struct boards *bop, struct solvedBoards *solvedp, 
-  int timeSupervision, int remainTime);
+int SolveAllBoards4(struct boards *bop, struct solvedBoards *solvedp);
+int SolveAllBoards1(struct boards *bop, struct solvedBoards *solvedp);
 
 
 
